package jp.gr.java_conf.skrb.http.nio;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;

import jp.gr.java_conf.skrb.http.ResponseHandler;

public class NioResponseHandler implements ResponseHandler {
    private static final Charset ISO_8859_1 = Charset.forName("ISO-8859-1");

    private SocketChannel channel;
    private String root;

    private FileChannel fileChannel;

    public NioResponseHandler(SocketChannel channel, String root) {
        this.channel = channel;
        this.root = root;
    }

    public void handle(String filename) {
        File file = new File(root, filename.replace('/', File.separatorChar));
        long length = file.length();

        if (!file.exists() || length == 0) {
            handleError(HttpURLConnection.HTTP_NO_CONTENT, filename + " doesn't exist.");
            return;
        }

        String filename2 = filename.toLowerCase();
        
        String type;
        if (filename2.endsWith(".html")
            || filename2.endsWith(".htm")) {
            type = "text/html";
        } else {
	    type = "application/octet-stream";
        }

        String header = makeHeader(type, length);
        CharBuffer cBuffer = CharBuffer.wrap(header);
        ByteBuffer buffer = ISO_8859_1.encode(cBuffer);

        try {
            channel.write(buffer);

            fileChannel = new FileInputStream(file).getChannel();
            fileChannel.transferTo(0, fileChannel.size(), channel);
        } catch (IOException ex) {
            ex.printStackTrace();
            try {
                if (fileChannel != null) {
                    fileChannel.close();
                }
                if (channel != null) {
                    channel.close();
                }
            } catch (IOException ex2) {
                ex2.printStackTrace();
            }
        }
    }

    public void handleError(int errorCode, String message) {
        try {
            StringBuilder builder = new StringBuilder();
            builder.append("HTTP/1.1 \r\n");
            builder.append(errorCode);
            builder.append(" ");
            builder.append(message);
            builder.append("\r\n\r\n");
            
            CharBuffer cBuffer = CharBuffer.wrap(builder.toString());
            ByteBuffer buffer = ISO_8859_1.encode(cBuffer);
            
            channel.write(buffer);
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                if (channel != null) {
                    channel.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    private String makeHeader(String type, long length) {
        StringBuilder builder = new StringBuilder();
        builder.append("HTTP/1.1 \r\n");
        builder.append("Content-type: ");
        builder.append(type);
        builder.append("\r\n");
        builder.append("Content-length: ");
        builder.append(length);
        builder.append("\r\n\r\n");

        return builder.toString();
    }
}
