package jp.gr.java_conf.skrb.http.stream;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.Socket;
import java.nio.charset.Charset;

import jp.gr.java_conf.skrb.http.RequestParser;

public class StreamRequestHandler implements Runnable {
    private static final Charset ISO_8859_1 = Charset.forName("ISO-8859-1");
    public static final int CHAR_SIZE = 4096;

    private Socket socket;
    private String root;
    private RequestParser parser;

    public StreamRequestHandler(Socket socket, String root, RequestParser parser) {
        this.socket = socket;
        this.root = root;
        this.parser = parser;
    }

    public void run() {
        try {
            Reader reader = new InputStreamReader(socket.getInputStream(),
                                                  ISO_8859_1);
            char[] chars = new char[CHAR_SIZE];;
            int length = 0;
            
            while (true) {
                int l = reader.read(chars, length, CHAR_SIZE);
                if (l < 0) {
                    break;
                } else {
                    length += l;
                    if (l < CHAR_SIZE) {
                        break;
                    } else {
                        char[] newChars = new char[length + CHAR_SIZE];
                        System.arraycopy(chars, 0, newChars, 0, length);
                        chars = newChars;
                    }
                }
            }
            
            parser.parse(new String(chars, 0, length),
                         new StreamResponseHandler(root, socket));
        } catch (IOException ex) {    
            if (socket != null) {
                new StreamResponseHandler(root, socket).handleError(HttpURLConnection.HTTP_BAD_REQUEST, "bad request");
            }
        } finally {    
            try {
                if (socket != null) {
                    socket.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
}
