package jp.gr.java_conf.skrb.http.stream;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.HttpURLConnection;
import java.net.Socket;

import jp.gr.java_conf.skrb.http.ResponseHandler;

public class StreamResponseHandler implements ResponseHandler {
    private String root;
    private Socket socket;

    public StreamResponseHandler(String root, Socket socket) {
        this.root = root;
        this.socket = socket;
    }

    public void handle(String filename) {
        File file = new File(root, filename.replace('/', File.separatorChar));
        long length = file.length();

        if (!file.exists() || length == 0) {
            handleError(HttpURLConnection.HTTP_NO_CONTENT, filename + " doesn't exist.");
            return;
        }

        String[] parts = file.getName().split("\\.");
        String ext = parts[parts.length - 1];
        String type;
            
        if (ext.equalsIgnoreCase("html")
            || ext.equalsIgnoreCase("htm")) {
            type = "text/html";
        } else {
	    type = "application/octet-stream";
        }
        
        OutputStream outputStream = null;
        InputStream inputStream = null;

        try {
            outputStream = socket.getOutputStream();
            inputStream = new FileInputStream(file);

            outputStream.write(makeHeader(type, length).getBytes());
            byte[] buffer = new byte[StreamRequestHandler.CHAR_SIZE];
            
            while(true) {
                int l = inputStream.read(buffer);
                if (l < 0) {
                    break;
                }
                outputStream.write(buffer, 0, l);
            }
            
            outputStream.flush();
        } catch (IOException ex) {
            handleError(HttpURLConnection.HTTP_BAD_REQUEST, "bad request");
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }

                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    public void handleError(int errorCode, String message) {
        PrintStream outputStream = null;
        try {
            outputStream = new PrintStream(socket.getOutputStream());
            outputStream.print("HTTP/1.1 ");
            outputStream.print(errorCode);
            outputStream.print(" ");
            outputStream.print(message);
            outputStream.print("\r\n\r\n");
            outputStream.flush();
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            if (outputStream != null) {
                outputStream.close();
            }
        }
    }

    private String makeHeader(String type, long length) {
        StringBuilder builder = new StringBuilder();
        builder.append("HTTP/1.1 \r\n");
        builder.append("Content-type: ");
        builder.append(type);
        builder.append("\r\n");
        builder.append("Content-length: ");
        builder.append(length);
        builder.append("\r\n");
        builder.append("\r\n");

        return builder.toString();
    }
}
